import os
from ..abstract_webtools import *
from .urlManager import *
from urllib.parse import urlparse
from abstract_utilities import *
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
import logging
import urllib3

# Suppress urllib3 warnings and debug logs
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
logging.getLogger("urllib3").setLevel(logging.WARNING)

# Suppress Selenium logs
logging.getLogger("selenium").setLevel(logging.WARNING)

import os
from selenium import webdriver
from selenium.webdriver.chrome.options import Options

# Setup Chrome options
chrome_options = Options()
chrome_options.binary_location = "/home/profiles/solcatcher/.cache/selenium/chrome/linux64/130.0.6723.58/chrome"
chrome_options.add_argument("--headless")  # Run in headless mode
chrome_options.add_argument("--no-sandbox")
chrome_options.add_argument("--disable-dev-shm-usage")
chrome_options.add_argument("--disable-gpu")
chrome_options.add_argument("--disable-software-rasterizer")
chrome_options.add_argument("--disable-extensions")
chrome_options.add_argument("--remote-debugging-port=9222")


class SingletonMeta(type):
    _instances = {}
    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            instance = super().__call__(*args, **kwargs)
            cls._instances[cls] = instance
        return cls._instances[cls]

class seleniumManager(metaclass=SingletonMeta):
    def __init__(self, url):
        if not hasattr(self, 'initialized'):  # Prevent reinitialization
            self.initialized = True
            parsed_url = urlparse(url)
            self.domain = parsed_url.netloc
            self.scheme = parsed_url.scheme
            self.base_url= f"{self.scheme}{self.domain}"
            self.site_dir = os.path.join(os.getcwd(), self.domain)
            os.makedirs(self.site_dir, exist_ok=True)
            self.drivers = {}
            self.page_type = []
    
    def get_url_to_path(self, url):
        url = eatAll(str(url), ['',' ','\n','\t','\\','/'])
        parsed_url = urlparse(url)
        if parsed_url.netloc == self.domain:
            paths = parsed_url.path.split('/')
            dir_path = self.site_dir
            for path in paths[:-1]:
                dir_path = os.path.join(dir_path, path)
                os.makedirs(dir_path, exist_ok=True)
            self.page_type.append(os.path.splitext(paths[-1])[-1] or 'html' if len(self.page_type) == 0 else self.page_type[-1])
            
            dir_path = os.path.join(dir_path, paths[-1])
            return dir_path

    def saved_url_check(self, url):
        path = self.get_url_to_path(url)
        return path

    def get_with_netloc(self, url):
        parsed_url = urlparse(url)
        if parsed_url.netloc == '':
            url = f"{self.scheme}://{self.domain}/{url.strip()}"
        return url

    def get_driver(self, url):
        if url and url not in self.drivers:
            chrome_options = Options()
            chrome_options.add_argument("--headless")
            driver = webdriver.Chrome(options=chrome_options)
            self.drivers[url] = driver
            driver.get(url)
        return self.drivers[url]
def normalize_url(url, base_url=None):
    """
    Normalize and resolve relative URLs, ensuring proper domain and format.
    """
    # If URL starts with the base URL repeated, remove the extra part
    manager = seleniumManager(url)
    base_url = manager.base_url
    if url.startswith(base_url):
        url = url[len(base_url):]

    # Resolve the URL against the base URL
    normalized_url = urljoin(base_url, url.split('#')[0])

    # Ensure only URLs belonging to the base domain are kept
    if not normalized_url.startswith(base_url):
        return None

    return normalized_url
# Function to get Selenium page source
def get_selenium_source(url):
    url_mgr = urlManager(url)
    if url_mgr.url:
        url = str(url_mgr.url)
        manager = seleniumManager(url)
        driver = manager.get_driver(url)
        try:
            # Get page source
            page_source = driver.page_source
            return page_source
        finally:
            # Don't quit the driver unless you're done with all interactions
            pass




from extention_list import get_extention,popular_extentions
from urllib.parse import urlparse, urljoin
from abstract_utilities import *
def try_request(url,timeout=None):
    if timeout == None:
        timeout= 5
    elif timeout == 0:
        timeout = None
    try:
        result = requests.get(url, timeout=timeout)  # Set timeout to 5 seconds
    except requests.exceptions.RequestException as e:
        print(f"Request failed for {url}: {e}")
        result = None
    return result
def is_result_200(result):
    try:
        if result.status_code == 200:
            return True
    except:
        return False
    return False
def url_to_pieces(url):
    """
    Split a URL into protocol, domain, path, and query components.
    Uses urlparse for robustness.
    """
    parsed_url = {'parsed':'', 'scheme':'', 'netloc':'', 'subdomain':'', 'domain':url,'extention':'', 'path':'', 'params':'', 'query':'', 'fragment':''}
    try:
        parsed = urlparse(url)
        parsed_url['parsed']= parsed
        parsed_url['scheme'] = parsed.scheme if parsed.scheme else ""
        parsed_url['netloc'] = parsed.netloc if parsed.netloc else ""
        parsed_url['path'] = parsed.path or ""
        parsed_url['params'] = parsed.params or ""
        parsed_url['query'] = parsed.query or ""
        parsed_url['fragment'] = parsed.fragment or ""
        if parsed_url['netloc'] == '' and parsed_url['path']:
            parsed_url['netloc'] = parsed_url['path']
            if '/' in parsed_url['path']:
                parsed_url['netloc'] = parsed_url['path'].split('/')[0]
                parsed_url['path'] = '/'+'/'.join(parsed_url['path'].split('/')[1:])
            else:
                parsed_url['path']=''
        if parsed_url['netloc']:
            if parsed_url['netloc'].startswith('www.'):
                parsed_url['subdomain']= 'www.'
                parsed_url['domain'] = parsed_url['netloc'][len('www.'):]
            else:
                parsed_url['domain'] = parsed_url['netloc']
            parsed_url.update(get_extention(parsed_url['domain']))
    except Exception as e:
        print(f'The URL {url} was not reachable: {e}')
    return parsed_url
def correct_domains(url):
    urls = [url]
    protocols = {'https':['','www.'],'http':['','www.'],'':['','www.']}
    parsed_url = url_to_pieces(url)
    scheme,subdomain,extentions = parsed_url['scheme'], parsed_url['subdomain'],make_list(parsed_url['extention'] or popular_extentions)
    subdomains = protocols.get(scheme)
    if subdomain in subdomains:
        subdomains.remove(subdomain)
    protocols[scheme] = subdomains
    for extention in extentions:
        link = f"{parsed_url['domain']}{extention}{parsed_url['path']}{parsed_url['params']}"
        for key,values in protocols.items():
            for value in values:
                new_link = f"{value}{link}"
                if key:
                    new_link = f"{key}://{new_link}"
                urls.append(new_link)
    return urls
def tryAllDomains(url):
    urls = correct_domains(url)
    for i, url in enumerate(urls):
        result = try_request(url)
        if is_result_200(result):
            return url
def tryDomain(url):
    request_mgr = requestManager(url)
    return request_mgr.source_code
url='thedailydialectics'
input(tryAllDomains(url))



import os
import requests
import os
import subprocess
import stat

def get_site(website, destination_dir, filename):
    # Ensure the directory exists
    os.makedirs(destination_dir, exist_ok=True)

    # Adjust directory permissions if needed (e.g. rwxr-xr-x -> 0o755)
    os.chmod(destination_dir, 0o755)

    # Construct the complete file path
    destination_path = os.path.join(destination_dir, filename)

    # Use curl to download the site
    # The example user-agent is arbitrary; you can change it to your needs
    os.system(
        f'curl -L --output "{destination_path}" '
        f'-H "User-Agent: Mozilla/5.0 (Windows NT 10.0; Win64; x64) '
        f'AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 '
        f'Safari/537.36" -H "Accept: */*" "{website}"'
    )

def download_site(website, destination_dir, filename):
    os.makedirs(destination_dir, exist_ok=True)
    os.chmod(destination_dir, 0o755)  # set directory permissions if needed

    destination_path = os.path.join(destination_dir, filename)

    # GET the resource
    response = requests.get(website, headers={
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
                      "AppleWebKit/537.36 (KHTML, like Gecko) "
                      "Chrome/91.0.4472.124 Safari/537.36",
        "Accept": "*/*"
    }, allow_redirects=True)

    # Raise an exception if the download fails
    response.raise_for_status()

    # Write content to file
    with open(destination_path, "wb") as f:
        f.write(response.content)
website = 'https://www.pornhub.com'
destination = '/home/computron/Documents/doge'
get_site(website,destination,'doge')



import re
import urllib.parse
import requests
from urllib.parse import urlparse, urlunparse, urljoin

from ...abstract_webtools import *

class urlManager:
    """
    Revised urlManager for managing and cleaning URLs.
    
    It splits URLs into their components, normalizes them (trimming spaces, lowercasing
    scheme and domain, removing default ports, and cleaning up paths), and then creates
    a list of potential variants (with/without www, http/https) so that a valid version
    can be determined.
    """
    def __init__(self, url=None, session=None):
        url = url or 'www.example.com'
        self._url = url
        self.session = session or requests
        self.clean_urls = self.clean_url(url)
        self.url = self.get_correct_url(clean_urls=self.clean_urls) or url
        self.protocol, self.domain, self.path, self.query = self.url_to_pieces(self.url)
        self.all_urls = []
    
    def url_to_pieces(self, url):
        """
        Split a URL into protocol, domain, path, and query components.
        Uses urlparse for robustness.
        """
        try:
            parsed = urlparse(url)
            protocol = parsed.scheme if parsed.scheme else None
            domain = parsed.netloc if parsed.netloc else None
            path = parsed.path or ""
            query = parsed.query or ""
        except Exception as e:
            print(f'The URL {url} was not reachable: {e}')
            protocol, domain, path, query = None, None, "", ""
        return protocol, domain, path, query

    def clean_url(self, url=None) -> list:
        """
        Normalize and clean the URL, then return a list of potential URL variants.
        
        This method:
          - Strips whitespace.
          - Adds a scheme (defaults to https) if missing.
          - Lowercases the scheme and domain.
          - Removes default ports.
          - Cleans up the path (removing duplicate slashes and trailing slash).
          - Generates variants with and without 'www', and with both http and https.
        """
        url = url or self._url
        url = url.strip()
        # Ensure the URL has a scheme
        if not re.match(r'https?://', url):
            url = 'https://' + url

        parsed = urlparse(url)
        scheme = parsed.scheme.lower()
        netloc = parsed.netloc.lower()
        # Remove default port numbers if present
        if ':' in netloc:
            host, port = netloc.split(':', 1)
            if (scheme == "http" and port == "80") or (scheme == "https" and port == "443"):
                netloc = host
        
        # Normalize the path: remove duplicate slashes and a trailing slash
        path = re.sub(r'//+', '/', parsed.path).rstrip('/')
        
        # Rebuild the cleaned URL without query or fragment
        cleaned_url = urlunparse((scheme, netloc, path, '', '', ''))
        
        variants = []
        # Add the primary variant
        variants.append(cleaned_url)
        # Generate a variant with/without 'www'
        if netloc.startswith('www.'):
            no_www = netloc[4:]
            variants.append(urlunparse((scheme, no_www, path, '', '', '')))
        else:
            variants.append(urlunparse((scheme, f"www.{netloc}", path, '', '', '')))
        
        # Also generate variants with the alternate scheme
        alt_scheme = 'http' if scheme == 'https' else 'https'
        for variant in list(variants):
            parsed_variant = urlparse(variant)
            alt_variant = urlunparse((alt_scheme, parsed_variant.netloc, parsed_variant.path, '', '', ''))
            variants.append(alt_variant)
        
        # Remove duplicates while preserving order
        seen = set()
        unique_variants = []
        for v in variants:
            if v not in seen:
                unique_variants.append(v)
                seen.add(v)
        return unique_variants

    def get_correct_url(self, url=None, clean_urls=None) -> str:
        """
        Attempts each URL variant by making an HTTP GET request.
        Returns the first variant that returns a 200 OK response.
        """
        if url is None and clean_urls is None:
            url = self._url
            clean_urls = self.clean_urls
        if url is not None and clean_urls is None:
            clean_urls = self.clean_url(url)
        elif url is None and clean_urls is not None:
            url = self._url

        for candidate in clean_urls:
            try:
                response = self.session.get(candidate, timeout=5)
                if response.status_code == 200:
                    return candidate
            except requests.exceptions.RequestException as e:
                print(f"Failed to reach {candidate}: {e}")
        return None

    def update_url(self, url):
        """
        Update the URL and refresh related attributes.
        """
        self._url = url
        self.clean_urls = self.clean_url(url)
        self.url = self.get_correct_url(clean_urls=self.clean_urls) or url
        self.protocol, self.domain, self.path, self.query = self.url_to_pieces(self.url)
        self.all_urls = []

    def get_domain(self, url=None):
        url = url or self.url
        return urlparse(url).netloc

    def url_join(self, base_url, path):
        """
        Joins a base URL with a relative path.
        """
        base_url = base_url.strip().rstrip('/')
        path = path.strip().lstrip('/')
        return f"{base_url}/{path}"

    @property
    def url(self):
        return self._url

    @url.setter
    def url(self, new_url):
        self._url = new_url

    def is_valid_url(self, url=None):
        """
        Check if the given URL is valid.
        """
        url = url or self.url
        parsed = urlparse(url)
        return bool(parsed.scheme) and bool(parsed.netloc)
    
    def make_valid(self, href, url=None):
        """
        Validate a href. If it's not already valid, join it with the base URL.
        """
        if self.is_valid_url(href):
            return href
        base = url or self.url
        new_link = urljoin(base, href)
        if self.is_valid_url(new_link):
            return new_link
        return False

    def get_relative_href(self, base, href):
        """
        For a relative href, join it with the base URL and strip any query or fragment.
        """
        joined = urljoin(base, href)
        parsed = urlparse(joined)
        clean_href = urlunparse((parsed.scheme, parsed.netloc, parsed.path, '', '', ''))
        return clean_href

    def url_basename(self, url=None):
        url = url or self.url
        path = urlparse(url).path
        return path.strip('/').split('/')[-1]

    def base_url(self, url=None):
        url = url or self.url
        match = re.match(r'https?://[^?#/]+/', url)
        if match:
            return match.group()
        return None

    def urljoin(self, base, path):
        return urljoin(base, path)

class urlManagerSingleton:
    _instance = None

    @staticmethod
    def get_instance(url=None, session=requests):
        if urlManagerSingleton._instance is None:
            urlManagerSingleton._instance = urlManager(url, session=session)
        elif urlManagerSingleton._instance.session != session or urlManagerSingleton._instance.url != url:
            urlManagerSingleton._instance = urlManager(url, session=session)
        return urlManagerSingleton._instance

def get_url(url=None, url_mgr=None):
    if not url and not url_mgr:
        return None
    if url:
        url_mgr = urlManager(url)
    return url_mgr.url

def get_url_mgr(url=None, url_mgr=None):
    if url_mgr is None and url:
         url_mgr = urlManager(url=url)
    if url_mgr and url is None:
        url = url_mgr.url
    return url_mgr



extentions=['.ac', '.academy', '.accountant', '.actor', '.agency', '.ai', '.airforce', '.am', '.apartments', '.archi', '.army', '.art', '.asia', '.associates', '.at', '.attorney', '.auction', '.audio', '.baby', '.band', '.bar', '.bargains', '.be', '.beer', '.berlin', '.best', '.bet', '.bid', '.bike', '.bingo', '.bio', '.biz', '.black', '.blackfriday', '.blog', '.blue', '.boston', '.boutique', '.br.com', '.build', '.builders', '.business', '.buzz', '.buz', '.ca', '.cab', '.cafe', '.camera', '.camp', '.capital', '.cards', '.care', '.careers', '.casa', '.cash', '.casino', '.catering', '.cc', '.center', '.ceo', '.ch', '.charity', '.chat', '.cheap', '.christmas', '.church', '.city', '.claims', '.cleaning', '.click', '.clinic', '.clothing', '.cloud', '.club', '.cn.com', '.co', '.co.com', '.co.in', '.co.nz', '.co.uk', '.coach', '.codes', '.coffee', '.college', '.com', '.com.co', '.com.mx', '.com.tw', '.community', '.company', '.computer', '.condos', '.construction', '.consulting', '.contact', '.contractors', '.cooking', '.cool', '.coupons', '.courses', '.credit', '.creditcard', '.cricket', '.cruises', '.cymru', '.cz', '.dance', '.date', '.dating', '.de', '.de.com', '.deals', '.degree', '.delivery', '.democrat', '.dental', '.dentist', '.desi', '.design', '.diamonds', '.diet', '.digital', '.direct', '.directory', '.discount', '.doctor', '.dog', '.domains', '.download', '.earth', '.eco', '.education', '.email', '.energy', '.engineer', '.engineering', '.enterprises', '.equipment', '.estate', '.eu', '.eu.com', '.events', '.exchange', '.expert', '.exposed', '.express', '.fail', '.faith', '.family', '.fans', '.farm', '.fashion', '.film', '.finance', '.financial', '.fish', '.fishing', '.fit', '.fitness', '.flights', '.florist', '.flowers', '.fm', '.football', '.forsale', '.foundation', '.fun', '.fund', '.furniture', '.futbol', '.fyi', '.gallery', '.games', '.garden', '.gay', '.gift', '.gifts', '.gives', '.glass', '.global', '.gmbh', '.gold', '.golf', '.graphics', '.gratis', '.green', '.gripe', '.group', '.gs', '.guide', '.guitars', '.guru', '.haus', '.healthcare', '.help', '.hiphop', '.hn', '.hockey', '.holdings', '.holiday', '.horse', '.host', '.hosting', '.house', '.how', '.immo', '.in', '.industries', '.info', '.ink', '.institue', '.insure', '.international', '.investments', '.io', '.irish', '.it', '.jetzt', '.jewelry', '.jp', '.jpn.com', '.juegos', '.kaufen', '.kim', '.kitchen', '.kiwi', '.la', '.land', '.lawyer', '.lease', '.legal', '.lgbt', '.li', '.life', '.lighting', '.limited', '.limo', '.link', '.live', '.llc', '.loan', '.loans', '.lol', '.london', '.love', '.ltd', '.luxury ', '.maison', '.managment', '.market', '.marketing', '.mba', '.me', '.me.uk', '.media', '.memorial', '.men', '.menu', '.miami', '.mobi', '.moda', '.moe', '.money', '.monster', '.mortgage', '.mx', '.nagoya', '.navy', '.net', '.net.co', '.network', '.news', '.ngo', '.ninja', '.nl', '.nyc', '.okinawa', '.one', '.ong', '.online', '.org', '.org.in', '.org.uk', '.partners', '.parts', '.party', '.pet', '.ph', '.photo', '.photography', '.photos', '.physio', '.pics', '.pictures', '.pink', '.pizza', '.pl', '.place', '.plumbing', '.plus', '.poker', '.press', '.pro', '.productions', '.promo', '.properties', '.property', '.pub', '.qpon', '.quebec', '.racing', '.realty', '.recipes', '.red', '.rehab', '.reisen', '.rent', '.rentals', '.repair', '.report', '.republican', '.rest', '.restaurant', '.review', '.reviews', '.rip', '.rocks', '.rodeo', '.run', '.sa.com', '.sale', '.sarl', '.sc', '.school', '.schule', '.science', '.se.net', '.services', '.sexy', '.sg', '.shiksha', '.shoes', '.shop', '.shopping', '.show', '.singles', '.site', '.ski', '.soccer', '.social', '.software', '.solar', '.solutions', '.soy', '.space', '.srl', '.store', '.stream', '.studio', '.study', '.style', '.supplies', '.supply', '.support', '.surf', '.surgery', '.systems', '.tattoo', '.tax', '.taxi', '.team', '.tech', '.technology', '.tel', '.tennis', '.theater', '.tienda', '.tips', '.today', '.tokyo', '.tools', '.tours', '.town', '.toys', '.trade', '.training', '.tv', '.tw', '.uk', '.uk.com', '.university', '.uno', '.us', '.us.com', '.vacations', '.vc', '.vegas', '.ventures', '.vet', '.viajes', '.video', '.villas', '.vip', '.vision', '.vodka', '.vote', '.voting', '.voyage', '.watch', '.webcam', '.website', '.wedding', '.wiki', '.win', '.wine', '.work', '.works', '.world', '.ws', '.wtf', '.xyz', '.yoga', '.za.com', '.zone']
popular_extentions = ['.com','.net','.org','.co','.us']
extentions = popular_extentions+[extention for extention in extentions if extention not in popular_extentions]
def get_extention(domain):
    domain_js = {"domain":domain,"extention":''}
    for extention in extentions:
        if domain.endswith(extention):
            domain_js["domain"] = domain[:-len(extention)]
            domain_js["extention"] = extention
            break
    return domain_js
